// Wrapper that exposes Scintilla's internal lexers as external lexers

// Inspired by the haskell external lexer - Credits to  Tobias Engvall - tumm at dtek dot chalmers dot se
// which was inspired by the caml external lexer - Credits to Robert Roessler - http://www.rftp.com


#include <ctype.h>
#include <string>

#include "Platform.h"

#include "PropSet.h"
#include "Accessor.h"
#include "KeyWords.h"
#include "Platform.h"

#include "PropSetSimple.h"

#include "ExternalLexer.h"
#include "WindowAccessor.h"

#include <windows.h>

extern void FoldPyDoc(unsigned int , int , int , WordList *[], Accessor &);
extern void FoldRbDoc(unsigned int , int , int , WordList *[], Accessor &);
extern void FoldCppDoc(unsigned int , int , int , WordList *[], Accessor &);

extern void ColourisePyDoc(unsigned int , int , int , WordList *[], Accessor &);
extern void ColouriseRbDoc(unsigned int , int , int , WordList *[], Accessor &);
extern void ColouriseCppDoc(unsigned int , int , int , WordList *[], Accessor &, bool);

enum LEXERS {
	LEX_PYTHON,
	LEX_RUBY,
	LEX_CPP
};

static const char* LexerNames[] = { 
	"python", 
	"ruby", 
	"cpp" 
};


static void InternalLexOrFold(int lexer, int foldOrLex, unsigned int startPos, int length,
	int initStyle, char *words[], WindowID window, char *props)
{
	// create and initialize a WindowAccessor (including contained PropSet)
	PropSetSimple ps;
	ps.SetMultiple(props);
	WindowAccessor wa(window, ps);

	// create and initialize WordList(s)
	int nWL = 0;
	for (; words[nWL]; nWL++) ;	// count # of WordList PTRs needed
	WordList** wl = new WordList* [nWL + 1];// alloc WordList PTRs
	int i = 0;
	for (; i < nWL; i++) {
		wl[i] = new WordList();	// (works or THROWS bad_alloc EXCEPTION)
		wl[i]->Set(words[i]);
	}
	wl[i] = 0;

	// call the folder/lexer 
	if (foldOrLex)
	{
		switch (lexer)
		{
		case LEX_PYTHON:
			FoldPyDoc(startPos, length, initStyle, wl, wa);
			break;
		case LEX_RUBY:
			FoldRbDoc(startPos, length, initStyle, wl, wa);
			break;
		default:
			FoldCppDoc(startPos, length, initStyle, wl, wa);
			break;
		}
	}
	else
	{
		switch (lexer)
		{
		case LEX_PYTHON:
			ColourisePyDoc(startPos, length, initStyle, wl, wa);
			break;
		case LEX_RUBY:
			ColouriseRbDoc(startPos, length, initStyle, wl, wa);
			break;
		default:
			ColouriseCppDoc(startPos, length, initStyle, wl, wa, true);
			break;
		}
	}

	// be sure to flush everything
	wa.Flush();

	// clean up before leaving
	for (i = nWL - 1; i >= 0; i--)
		delete wl[i];
	delete [] wl;
}

extern "C" __declspec(dllexport) 
void EXT_LEXER_DECL Lex(unsigned int lexer, unsigned int startPos, int length, int initStyle,
                        char *words[], WindowID window, char *props)
{
	InternalLexOrFold(lexer, 0, startPos, length, initStyle, words, window, props);
}

extern "C" __declspec(dllexport) 
void EXT_LEXER_DECL Fold (unsigned int lexer, unsigned int startPos, int length, int initStyle,
                        char *words[], WindowID window, char *props)
{
	InternalLexOrFold(lexer, 1, startPos, length, initStyle, words, window, props);
}

extern "C" __declspec(dllexport) 
int EXT_LEXER_DECL GetLexerCount()
{
   return (sizeof(LexerNames)/sizeof(*LexerNames));
}

extern "C" __declspec(dllexport) 
void EXT_LEXER_DECL GetLexerName(unsigned int Index, char *name, int buflength)
{
   if (buflength > 0) {
      buflength--;
      int n = strlen(LexerNames[Index]);
      if (n > buflength)
         n = buflength;
      memcpy(name, LexerNames[Index], n), name[n] = '\0';
   }
}

